#ifndef BOOK_H
#define BOOK_H

#include <string>
#include <vector>
#include "LibraryItem.h"
#include "HoldEvent.h"
#include "LoanEvent.h"
#include "Clock.h"
using namespace std;

class LoanEvent;
class HoldEvent;


// A class inherited from class LibraryItem to represent and 
// manipulate a book from book collection

class Book : public LibraryItem
{
private:
	LoanEvent *_Book_Loan;
	HoldEvent *_hold_event;
	string _Author_Name;
	Clock *loan_due_date;
	int _loan_length;

public:
	Book();
	// ---The current book is undefined
	// An empty Book object has been created---
	// Pre :True
	// Post:True

	Book(string BarcodeNumber, string title, string Author_Name, int loan_length);
	// ---Initialise the book object with barcode, title, author and the loan length---
	// Pre :True
	// Post:GetBarcodeNumber()==BarcodeNumber && GetTitle()==title && _Author_Name=Author_Name
	//      && _loan_length=loan_lenght

	void Lend(LoanEvent *A_Book_Loan);
	// ---Set the due date for each book and set it to be loaned out---
	// Pre   :A_Book_Loan=NULL
	// Post  :this->GetLoanLength() && this->GetLoanDueDate() implies A_Book_Loan!=NULL

	void Return();
	//---The particular book has been returned and determine whether it is on hold 
	// 	by other member or it is overdue---
	// Pre   :True
	// Post  :NOT(OnHold()) && NOT(IsLoanOverdue) && NOT (OnLoan()) implies Book_Loan=NULL


	bool OnLoan();
	// ---return true if and only if the particular book is on loan---
	// Pre   :True
	// Post  :return _Book_Loan!=NULL


	bool OnHold();
	// ---return true if and only if the particular book is on hold---
	// Pre   :True
	// Post	 :return _hold_event!=NULL

	bool IsLoanOverdue();
	//---return true if and only if the book is overdue---
	// Pre :True
	// Post:return loan_due_date->Is_earlier_than_present_time())

	void Hold(HoldEvent *Book_Hold_Event);
	//---Set the book to be held by member---
	// pre   :Book_Hold_Event=NULL
	// Post  :if this->OnLoan() then loan_due_date=temp_clock.GetPresentDay()+2 
	//  	  else loan_due_date=temp_clock.GetPresentDay()+1
	//	  implies Book_Hold_Event!=NULL

	void CancelHold();
	// ---To cancel the book held---
	// pre   :hold_event!=NULL
	// Post  :hold_event=NULL

	string GetAuthorName()const;
	//---The author name has been returned as a string---
	// Pre   :True
	// Post  :return _Author_Name & _Author_Name'=_Author_Name

	int GetLoanLength()const;
	//---The loan length for each book has been returned---
	// Pre   :True
	// Post  :return loan_length & loan_length'=loan_length
	// invariant: loan_length>0

	LoanEvent* GetBookLoanEvent();
	//---To return the book involved in the loan to LoanEvent class----
	// Pre   :True
	// Post  :return _Book_Loan

	HoldEvent* GetBookHoldEvent();
	//---To return the book involved in the hold to HoldEvent class---
	// Pre   :True
	// Post  :return hold_event

	Clock *GetLoanDueDate();
	//---To return the book's due date to Clock class---
	// Pre   :True
	// Post  :return loan_due_date
		
	void Output();
	//--- Output the book on a single line ot standard output---
	// Pre   :A book has been selected.
	// Post  :OUTPUTS(LibraryItem::Output()) && GetAuthorName()

};

#endif