#ifndef BOOKCOLLECTION_H
#define BOOKCOLLECTION_H
#include <vector>
#include <string>
#include "Book.h"
using namespace std;


// A class used to represent and manipulate the collection of books
// in the library

class BookCollection
{
private:
	vector <Book > _book;	

public:
	BookCollection();
	// ---The current collection of book is undefined---
	// Pre   :True
	// Post  :True 

	void AddBook(string Book_Barcode, string title, string author_name, int loan_length);
	//--- This function adds a new book to the BookCollection---
	// Pre   :Book_Barcode,title,author_name,loan_length!=NULL
	// Post  :newBook IsIn BookCollection
	
	void RemoveBook(string book_barcode);
	//---Remove a book by its barcode number, if the barcode match, the book is removed---
	// Pre   :True
	// Post  :Book->GetBarcodeNumber()==book_barcode && NOT(Book IsIn BookCollection)

	Book *GetBookByBarcode(string book_barcode);
	// ---Search a book by its barcode number, if the barcode match, return a pointer to the Book else return NULL---
	// Pre   :True
	// Post  :Book->GetBarcodeNumber() == book_barcode &&
    //           (Book IsIn BookCollection &&
    //            return Book)
    //	      OR
    //           return NULL

	BookCollection *GetBookByTitle(string book_title);
	//---Search a book by its name, if the name matched, return a pointer to the BookCollection else return NULL---
	// Pre   :True
	// Post  :Book->GetTitle() == book_title &&
    //           (Book IsIn BookCollection &&
    //            return Book)
	//	      OR
    //           return NULL

	BookCollection *GetBookByAuthor(string authorname);
	//---Search a book by its author name, if the author name matched, return a pointer to BookCollection else return NULL---
	// Pre   :True	
	// Post  :Book->GetAuthorName() == authorname &&
    //           (Book IsIn BookCollection &&
    //            return Book)
	//  	  OR
    //           return NULL

	BookCollection *GetHoldFreeBookByTitle(string title);
	// --- To check whether there is duplicate copies of the book with same title, if yes, return a pointer to BookCollection else return NULL---
	// Pre   :True
	// Post  :Book->GetTitle()==title && !=OnHold() &&
    //           (Book IsIn BookCollection &&
    //            return Book)
	//	      OR
    //           return NULL

	void DisplayMemberWithOverdueBooks();
	//---To display a list of member with overdue books---
	// Pre	  : True
	// Post	  : Book->IsLoanOverdue()==true && OUTPUTS( details of member)

	
	void Output()const;
	//---To display all the book details on the standard output---
	// Pre   :True
	// Post  :for_all Book, Book IsIn BookCollection : OUTPUT Book
};


#endif