#ifndef CD_H
#define CD_H

#include <string>
#include <vector>
#include "LoanEvent.h"
#include "HoldEvent.h"
#include "LibraryItem.h"
#include "Clock.h"
using namespace std;

class HoldEvent;
class LoanEvent;


// A class inherited from class LibraryItem to represent and 
// manipulate a CD from CD collection

class CD : public LibraryItem
{
private:
	string _Artist_Name;
	string _Play_Time;
	int _loan_length;
	Clock *loan_due_date;
	LoanEvent *_CD_Loan;
	HoldEvent *_hold_event;

public:
	CD();
	// ---The current CD is undefined. An empty CD object has been created---
	// Pre   :True
	// Post  :True
	
	CD(string title, string BarcodeNumber, string Artist_Name, string Play_Time, int loan_length);
	//---Initialise all the member data and pass it as parameters---
	// Pre   :True
	// Post  :GetTitle()==title && GetBarcodeNumber()==BarcodeNumber 
	//		  && _Artist_Name=Artist_Name 
	//        && _Play_Time=Play_Time
	//     	  && _loan_length=loan_length

	
	void Lend(LoanEvent *A_CD_Loan);
	//---Set the due date for each CD and set it to be loaned out---
	// Pre   :A_CD_Loan=NULL
	// Post  :this->GetLoanLength() && this->GetLoanDueDate() implies A_CD_Loan!=NULL

	void Return();
	//---The particular CD has been returned and determine whether it is on hold by other member or it is overdue---
	// Pre	 :True
	// Post  :NOT(OnHold()) && NOT(IsLoanOverdue) && NOT (OnLoan()) implies CD_Loan=NULL
	

	bool OnLoan();
	// ---return true if and only if the particular CD is on loan---
	// Pre   :True
	// Post  :return _CD_Loan!=NULL
	
	bool OnHold();
	// ---return true if and only if the particular CD is on hold---
	// Pre   :True
	// Post	 :return _hold_event!=NULL

	bool IsLoanOverdue();
	//---return true if and only if the CD is overdue---
	// Pre :True
	// Post:return loan_due_date->Is_earlier_than_present_time())

	void Hold(HoldEvent *CD_Hold_Event);
	//---Set the CD to be held by member---
	// pre   :CD_Hold_Event=NULL
	// Post  :if this->OnLoan() then loan_due_date=temp_clock.GetPresentDay()+2 
	//  	  else loan_due_date=temp_clock.GetPresentDay()+1
	//		  implies CD_Hold_Event!=NULL

	void CancelHold();
	// ---To cancel the CD held---
	// pre   :hold_event!=NULL
	// Post  :hold_event=NULL

	string GetArtistName();
	//---The artist name has been returned as a string---
	// Pre   :True
	// Post  :return _Artist_Name & _Artist_Name'=_Artist_Name

	string GetPlayTime();
	//---The CD play time has been returned as a string---
	// Pre	 :True
	// Post  :return _Play_Time & _Play_Time'=_Play_Time

	int GetLoanLength()const;
	//---The loan length for each CD has been returned---
	// Pre   :True
	// Post  :return loan_length & loan_length'=loan_length
	// invariant: loan_length>0

	LoanEvent* GetCDLoanEvent();
	//---To return the CD involved in the loan to LoanEvent class---
	// Pre   :True
	// Post  :return _CD_Loan

	HoldEvent* GetCDHoldEvent();
	//---To return the CD involved in the hold to HoldEvent class---
	// Pre   :True
	// Post  :return _hold_event

	Clock *GetLoanDueDate();
	//---To return the CD's due date to Clock class---
	// Pre   :True
	// Post  :return loan_due_date

	void Output();
	//--- Output the CD on a single line ot standard output---
	// Pre   :A CD has been selected.
	// Post  :OUTPUTS(LibraryItem::Output()) && GetArtistName() && GetPlayTime()

};

#endif