#ifndef CLOCK_H
#define CLOCK_H

// Clock.h: interface for the Clock class. Clock class encapsulate
//			   all the operations related to time management 
//
//////////////////////////////////////////////////////////////////////
#include <time.h>
#include "Date.h"
#include "Time.h"

class Clock
{
private:
	Time _time;
	Date _date;

private:
	tm* GetPresentTime();

public:
	
	Clock();
	//---Create a Clock object---
	// Pre : True
	// Post: True
	
	Clock(int year, int month, int day, int hour, int minute);
	//---Initialise all the member data and initialise them to a valid value---
	// Pre : True
	// Post:(year% 400==0 implies month February==29 && month++) 
	//		&& (day>31 implies month++ && month<=12) && (hour>=24 implies day++ && day<=31)
	//      && (minute>=60 implies hour++ && hour<24)  

	virtual bool Is_same_as_present_time();
	//---return true if and only if the date and time is same as the present date and time---
	// Pre : True
	// Post: returns (_date'==system_clock._date) && (_time'==system_clock._time)
		
    virtual bool Is_earlier_than_present_time(); 
	//---return true if and only if the date and time is earlier than the present date and time---
	// Pre : True
	// Post:returns (_date'<system_clock._date) && (_time'<system_clock._time)

	virtual bool Is_later_than_present_time(); 
	//---return true if and only if the date and time is later then the present date and time---
	// Pre : True
	// Post: returns (_date'>system_clock._date) && (_time'>system_clock._time)

	void Output() const;
	//---Display the date details and the time details---
    // Pre  :  True
    // Post : for_all _date && for_all _time:OUTPUT DATE && OUTPUT TIME

	virtual int GetPresentYear();
	//---To get the present year details from the BIOS and return it to the Clock class---
	// Pre  :True
	// Post : tm_year'==tm_year+1900


	virtual int GetPresentMonth();
	////---To get the present month details from the BIOS and return it to the Clock class---
	// Pre  :True
	// Post : tm_mon'==tm_mon+1

	virtual int GetPresentDay();
	//---To get the present day from the BIOS and return it to the Clock class---
	// Pre  :True
	// Post : tm_mday'==tm_mday


	virtual int GetPresentHour();
	//---To get the present hour from the BIOS and return it to the Clock class---
	// Pre  :True
	// Post : tm_hour'==tm_hour


	virtual int GetPresentMinute();
	//---To get the present minute from the BIOS and return it to the Clock class---
	// Pre  :True
	// Post : tm_min'==tm_min


	virtual ~Clock();
	//---Destroy the Clock object----
	// Pre  :True
	// Post : True
};


#endif