#ifndef LIBRARY_H
#define LIBRARY_H

#include <string>
#include <vector>
#include <iostream>
#include "Book.h"
#include "BookCollection.h"
#include "Member.h"
#include "Membership.h"
#include "Loans.h"
#include "JournalCollection.h"
#include "CD.h"
#include "CDCollection.h"
#include "Journal.h"
#include "Video.h"
#include "VideoCollection.h"
#include "HoldItems.h"
using namespace std;

class Library
{

public:

	Library();
	// Pre: True
	// Post : True

	void BorrowItem();
	// Pre: _current_member!=NULL
	// Post :_current_book->BorrowBook() implies borrow book
	//	     _current_CD->BorrowCD() implies borrow CD
	//	     _current_video->BorrowVideo() implies borrow video
  
	void ReturnItem();
	// Pre : _current_member!=NULL
	// Post: _current_book->ReturnBook() implies return book
	//	     _current_CD->ReturnCD() implies return CD
	//	     _current_video->ReturnVideo() implies return video


	void HoldItem();
	// Pre : _current_member!=NULL
	// Post: _current_book->HoldBook() implies hold book
	//	     _current_CD->HoldCD() implies hold CD
	//	     _current_video->HoldVideo() implies hold video


	void CancelHoldItem();
	// Pre : _current_member!=NULL
	// Post: _current_book->CancelHoldBook() implies cancel hold book
	//	     _current_CD->CancelHoldCD() implies cancel hold CD
	//	     _current_video->CancelHoldVideo() implies cancel hold video

	void AddMember();
	//Pre  : _current_member!=NULL
	//Post : NewMember IsIn Membership

	void RemoveMember();
	//Pre  : _current_member!=NULL
	//Post : _current_member=NULL && NOT ( Member IsIn Membership)

	void AddBook();
	//Pre  :_current_book!=NULL
	//Post : Book IsIn BookCollection

	void AddCD();
	//Pre  :_current_cd!=NULL
	//Post : CD IsIn CDCollection

	void AddJournal();
	//Pre  : _current_journal!=NULL
	//Post: Journal IsIn JournalCollection

	void AddVideo();
	//Pre  : _current_video!=NULL
	//Post: Video IsIn VideoCollection

	void RemoveItem();
	// Pre : _current_book!=NULL, _current_journal!=NULL,
	//       _current_cd!=NULL,_current_video!=NULL
	// Post: (_current_book=NULL,_current_cd=NULL,
	//		 _current_video=NULL,_current_journal=NULL) &&
	//		 NOT (Book IsIn BookCollection) && NOT (Journal IsIn JournalCollection)
	//		 && Not(Video IsIn VideoCollection) && NOT( CD IsIn CDCollection)

	void SelectMember();
	// Pre : (_current_member!=NULL) && selection==1 OR selection==2
	// Post : OUTPUTS( details of the member match the criteria)

	void SelectItem();
	// Pre: True
	// Post: OUTPUTS( details of the item match the criteria)

	void DisplayMemberInfo();
	//Pre : _current_member!=NULL
	//Post : _current_member->Output()

	void DisplayItemInfo();
	//Pre : ITEM !=NULL WHERE ITEM INCLUDES book,journal, CD and video
	//Post: ITEM->Output()

	void DisplayMainMenu();
	// Pre: True
	// Post : DISPLAYS ( details of the library menu)

	void OnlineCatalog();
	// Pre : selection >=1 && selection <=5
	//Post : OUTPUTS( details from selection entered)


private:

	Membership *_library_members;
	BookCollection *_library_books;
	CDCollection *_library_cds;
	JournalCollection *_library_journals;
	VideoCollection *_library_videos;

	LoanEvent* _current_loan_event;
	HoldEvent* _current_hold_event;

	Loans* _loan_events;
	HoldItems* _hold_events;

	Member *_current_member;

	Book *_current_book;
	CD *_current_cd;
	Journal *_current_journal;
	Video *_current_video;


private:

	void BorrowBook();
	//Pre: _current_book!=NULL && !_current_book->OnHold()
	//		&& !_current_book->OnLoan() && !_current_member->IsLoanOverdue()
	//Post: _current_member->Borrow() && _current_member->Output() implies Book has been borrowed

	void BorrowCD();
	//Pre: _current_cd!=NULL && !_current_cd->OnHold()
	//		&& !_current_cd->OnLoan() && !_current_member->IsLoanOverdue()
	//Post: _current_member->Borrow() && _current_member->Output() implies CD has been borrowed

	void BorrowVideo();
	//Pre: _current_video!=NULL && !_current_video->OnHold()
	//		&& !_current_video->OnLoan() && !_current_member->IsLoanOverdue()
	//Post: _current_member->Borrow() && _current_member->Output() implies Video has been borrowed
	
	void ReturnBook();
	// Pre :_current_book->OnLoan()==true && _current_member!=NULL 
	// Post: NOT ( Book IsIn LoanEvent) && _current_member->Output() implies book has been returned
	
	void ReturnCD();
	// Pre :_current_cd->OnLoan()==true && _current_member!=NULL 
	// Post: NOT ( CDIsIn LoanEvent) && _current_member->Output() implies CD has been returned
	

	void ReturnVideo();
	// Pre :_current_video->OnLoan()==true && _current_member!=NULL 
	// Post: NOT ( Video IsIn LoanEvent) && _current_member->Output() implies video has been returned
	
	void HoldBook();
	// Pre : _current_member!=NULL && !_current_book->OnHold() && !_current_member->IsLoanOverdue()
	// Post: (Book IsIn HoldEvent) && _current_member->Output() 

	void HoldCD();
	// Pre : _current_member!=NULL && !_current_cd->OnHold() && !_current_member->IsLoanOverdue()
	// Post: (CD IsIn HoldEvent) && _current_member->Output() 

	void HoldVideo();
	// Pre : _current_member!=NULL && !_current_video->OnHold() && !_current_member->IsLoanOverdue()
	// Post: (Video IsIn HoldEvent) && _current_member->Output() 

	void CancelHoldBook();
	// Pre : _current_member!=NULL && _current_book->OnHold()==true
	//Post : NOT ( Book IsIn HoldEvent) && _current_member->Output()

	void CancelHoldCD();
	// Pre : _current_member!=NULL && _current_cd->OnHold()==true
	//Post : NOT (CDIsIn HoldEvent) && _current_member->Output()

	void CancelHoldVideo();
	// Pre : _current_member!=NULL && _current_video->OnHold()==true
	//Post : NOT ( Video IsIn HoldEvent) && _current_member->Output()

	void RemoveBook();
	//Pre : _current_book!=NULL 
	//Post : NOT ( Book IsIn BookCollection) 

	void RemoveCD();
	//Pre : _current_cd!=NULL 
	//Post : NOT (CD IsIn CDCollection)

	void RemoveJournal();
	//Pre : _current_journal!=NULL 
	//Post : NOT ( Journal IsIn JournalCollection)

	void RemoveVideo();
	//Pre : _current_video!=NULL 
	//Post : NOT ( Video IsIn VideoCollection)

	void SearchBook();
	//Pre : selection>=1 && selection <=3
	//Post: OUTPUTS ( details of the book match the search criteria)

	void SearchCD();
	//Pre : selection>=1 && selection <=3
	//Post: OUTPUTS ( details of the CD match the search criteria)


	void SearchJournal();
		//Pre : selection>=1 && selection <=3
	//Post: OUTPUTS ( details of the journal match the search criteria)

	void SearchVideo();
	//Pre : selection>=1 && selection <=3
	//Post: OUTPUTS ( details of the video match the search criteria)


	void DisplayMemberWithOverdueItem();
	//Pre : _current_member!=NULL
	//Post: _current_member->Output() implies that the member is found

	void AddItemMenu();
	//Pre : selection>=1 && selection<=6
	//Post : OUTPUTS( details of the item to be added)
};

#endif;


