#ifndef MEMBER_H
#define MEMBER_H

#include <string>
#include <vector>
#include "LoanEvent.h"
#include "HoldEvent.h"
using namespace std;

class LoanEvent;
class HoldEvent;
class Member
{
public:
	Member();
	//---The current member is undefined. An empty member object has 
	//   been created---
	// Pre :True
	// Post:A Member object has been constructed

	Member(string Member_Name, string Address, string Phone_Number, string E_Mail, int Member_Library_Number);
	//---The memebr data has been initialised and pass as parameter to create a member object---
	// Pre :True
	// Post:All member data have been initialised & member object has been created

	string GetName();
	//---To return a member name---
	// Pre :True
	// Post:returns _Member_Name

	int GetLibraryNumber();
	//---To return a member library number---
	// Pre :True
	// Post:returns _Member_Library_Number

	void Borrow(LoanEvent *A_Member_Loan);
	//---To add a new loan event to member collection of loan events---
	// Pre :A_Member_Loan!=NULL
	// Post:(ItemLoan IsIn LoanEvent) && LoanEvent_For_A_Member'=LoanEvent_For_A_Member+1

	void Return(string barcode);
	//---Scan the barcode of the item returned and destroy associate loan event for the member---
	// Pre :A_Member_Loan!=NULL
	// Post:NOT (ItemLoan IsIn LoanEvent) && LoanEvent_For_A_Member'=LoanEvent_For_A_Member-1

	LoanEvent* GetMemberLoanEvent(Book* book);
	//---takes a pointer to the Book as parameter and return pointer to LoanEvent---
	// Pre: book!=NULL
	// Post:returns _LoanEvent_For_A_Member 
	// 	OR returns NULL

	HoldEvent* GetMemberHoldEvent(Book* book);
	//---takes a pointer to the Book as parameter and return pointer to HoldEvent---
	// Pre: book!=NULL
	// Post: returns _HoldEvent_For_A_Member
	//	OR returns NULL

	LoanEvent* GetMemberLoanEvent(CD* cd);
	//---takes a pointer to the CD as parameter and return pointer to LoanEvent---
	// Pre: CD!=NULL
	// Post: returns _LoanEvent_For_A_Member
	//	OR returns NULL

	HoldEvent* GetMemberHoldEvent(CD* cd);
	//---takes a pointer to the CDas parameter and return pointer to HoldEvent---
	// Pre: CD!=NULL
	// Post: returns _HoldEvent_For_A_Member
	//	OR returns NULL

	LoanEvent* GetMemberLoanEvent(Video* video);
	//---takes a pointer to the Video as parameter and return pointer to LoanEvent---
	// Pre:video!=NULL
	// Post: returns _LoanEvent_For_A_Member
	//	OR returns NULL

	HoldEvent* GetMemberHoldEvent(Video* video);
	//---takes a pointer to the Video as parameter and return pointer to HoldEvent---
	// Pre: video!=NULL
	// Post: returns _HoldEvent_For_A_Member
	//	OR returns NULL

	void Hold(HoldEvent *hold_event);
	//---Add a new hold event to member collection of hold events---
	// Pre :hold_event!=NULL
	// Post:(ItemHold IsIn HoldEvent) && HoldEvent_For_A_Member'=HoldEvent_For_A_Member+1

	void CancelHold(string barcode);
	//---Destroy the member collection of hold events after the hold has been cancelled---
	// Pre :hold_event!=NULL
	// Post:NOT( ItemHold IsIn HoldEvent) && HoldEvent_For_A_Member'=HoldEvent_For_A_Member-1

	bool IsLoansOverdue();
	//---return true if and only if the item is overdue---
	// Pre: True
	// Post: 

	void Output();
	//---To output the details of all the member and the details of the items that held or loaned by member---
	// Pre :True
	// Post: OUTPUTS (Member details && details of Items On Hold 
	//       && details of Items On Loan)


private:
	string _Member_Name;
	string _Address;
	string _Phone_Number;
	string _E_Mail;
	int _Member_Library_Number;
	vector <LoanEvent> _LoanEvent_For_A_Member;
	vector <HoldEvent> _HoldEvent_For_A_Member;
};

#endif
