#ifndef already_included_month
#define already_included_month

#include "Year.h"
#include <string>
using namespace std;

enum Months {Jan=1,Feb,Mar,Apr,May,Jun,Jul,Aug,Sep,Oct,Nov,Dec};


// This class is designed to represent a particular month. That is, it is not
// not just a month number or name, but also the year in which that particular
// month occurred. So, for example, July 1997 and July 1998 would need to be
// represented by two different Month objects.

class Month
{
private:
    Year _year;
    int _month_nr;

public:
	Month();
	//---create a Month object---
	// Pre:  True
	// Post: True

    Month(int month_nr, int year_nr);
	//---Initilaise the member data and pass them as parameters to create a Month object---
    // Pre:  month_nr is a valid month of the year.
    // Post: A month object has been constructed based on the specified
    //       information.

    Month(int month_nr, Year year);
	//---Initilaise the member data and pass them as parameters to create a Month object---
    // Pre:  month_nr is a valid month of the year.
    // Post: A month object has been constructed based on the specified
    //       information.

    void Output() const;
	//---Output details of month and year to standard output---
    // Pre:  True
    // Post: OUTPUTS _month_nr && for_all _year: OUTPUT Year

    string Name() const;
	//---returns full name for each month such as Jan for January---
    // Pre:  True
    // Post: returns (_month's FULL NAME)

    bool Is_same_as(const Month &other_month) const;
	//---return true if and only if the month is same as other month---
    // Pre:  True
    // Post:returns (_year'==other _month._year) &&( _month'==other _month._month)
    
    bool Is_earlier_than(const Month &other_month) const;
	//---return true if and only if the month is earlier than other month---
    // Pre:  True
    // Post:returns (_year'<other _month._year) &&( _month'<other _month._month)
    

    bool Is_later_than(const Month &other_month) const;
	//---return true if and only if the month is later than other month---
    // Pre:  True
    // Post:returns (_year'>other _month._year) &&( _month'>other _month._month)
    
    int Days_in_month() const;
	//---returns number of days in a month, by default return 31 days---
    // Pre:  True
    // Post: returns 31 BY DEFAULT && (_year.Is_leap_year==true implies _nr_days ==29 &&         //       _month==Feb)

    Month Next_month() const;
	//---returns the next month after the end of the month---
    // Pre: True
    // Post: returns (_month_nr'==_month_nr+1 && _year'==_year && _month_nr<Dec)
 	//		 OR returns (_month_nr'==Jan && _year'==_year+1)
};

#endif