#ifndef VIDEO_H
#define VIDEO_H

#include <string>
#include <vector>
#include "LibraryItem.h"
#include "LoanEvent.h"
#include "HoldEvent.h"
#include "Clock.h"
using namespace std;


/*enum Ratings
{
	R=1,
	MA=2,
	G=3
};
*/
class LoanEvent;
class HoldEvent;

class Video:public LibraryItem
{
private:
	string _Director_Name;
	string _rating;
	int _loan_length;
	Clock* loan_due_date;
	HoldEvent* _hold_event;
	LoanEvent* _Video_Loan;


public:
	Video();
	//---The current video is undefined. An empty video object has been created---
	// Pre :True
	// Post:True

	Video(string Video_Title, string Video_Barcode_Number, string Director_Name , string rating, int loan_length);
	//---Initialise all the member data and pass it as parameters---
	// Pre :True
	// Post:GetBarcodeNumber()==BarcodeNumber && GetTitle()==title && _Director_Name=Director_Name
	//      && _rating=rating && _loan_length=loan_length

	void Lend(LoanEvent *A_Video_Loan);
	//---Set the due date for each video and set it to be loaned out---
	// Pre   :A_Video_Loan=NULL
	// Post  :this->GetLoanLength() && this->GetLoanDueDate() implies A_Video_Loan!=NULL


	void Return();
	//---The particular video has been returned and determine whether it is on hold by other member or it is overdue---
	// Pre   :True
	// Post  :NOT(OnHold()) && NOT(IsLoanOverdue) && NOT (OnLoan()) implies Video_Loan=NULL
	
	bool OnLoan();
	// ---return true if and only if the particular video is on loan---
	// Pre   :True
	// Post  :return _Video_Loan!=NULL

	bool OnHold();
	// ---return true if and only if the particular video is on hold---
	// Pre   :True
	// Post	 :return _hold_event!=NULL

	bool IsLoanOverdue();
	//---return true if and only if the video is overdue---
	// Pre :True
	// Post:return loan_due_date->Is_earlier_than_present_time())

	void Hold(HoldEvent *Video_Hold_Event);
	//---Set the video to be held by member---
	// pre   :Video_Hold_Event=NULL
	// Post  :if this->OnLoan() then loan_due_date=temp_clock.GetPresentDay()+2 
	//  	  else loan_due_date=temp_clock.GetPresentDay()+1
	//	  implies Video_Hold_Event!=NULL

	void CancelHold();
	// ---To cancel the video held---
	// pre   :hold_event!=NULL
	// Post  :hold_event=NULL

	string GetDirectorName()const;
	//---The director name has been returned as a string---
	// Pre   :True
	// Post  :return _Director_Name & _Director_Name'=_Director_Name

	int GetLoanLength()const;
	//---The loan length for each video has been returned---
	// Pre   :True
	// Post  :return loan_length & loan_length'=loan_length
	// invariant: loan_length>0

	string GetRating()const;
	//---The video's rating eg.MA,G has been returned as a string---
	// Pre   :True
	// Post  :return _rating & _rating'=_rating

	LoanEvent* GetVideoLoanEvent();
	//---To return the video involved in the loan to LoanEvent class----
	// Pre   :True
	// Post  :return _Video_Loan

	HoldEvent* GetVideoHoldEvent();
	//---To return the video involved in the hold to HoldEvent class---
	// Pre   :True
	// Post  :return hold_event

	Clock *GetLoanDueDate();
	//---To return the video's due date to Clock class---
	// Pre   :True
	// Post  :return loan_due_date

	void Output();
	//--- Output the video on a single line ot standard output---
	// Pre   :A video has been selected.
	// Post  :OUTPUTS(LibraryItem::Output()) && GetDirectorName() & GetRating()
};

#endif


